// Code generated by go-swagger; DO NOT EDIT.

// Copyright 2022 The Sigstore Authors.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

package timestamp

import (
	"context"
	"io"
	"time"

	"github.com/go-openapi/runtime"
	httptransport "github.com/go-openapi/runtime/client"
	"github.com/go-openapi/strfmt"
)

// New creates a new timestamp API client.
func New(transport runtime.ContextualTransport, formats strfmt.Registry) ClientService {
	return &Client{transport: transport, formats: formats}
}

// New creates a new timestamp API client with basic auth credentials.
//
// It takes the following parameters:
// - host: http host (github.com).
// - basePath: any base path for the API client ("/v1", "/v3").
// - scheme: http scheme ("http", "https").
// - user: user for basic authentication header.
// - password: password for basic authentication header.
func NewClientWithBasicAuth(host, basePath, scheme, user, password string) ClientService {
	transport := httptransport.New(host, basePath, []string{scheme})
	transport.DefaultAuthentication = httptransport.BasicAuth(user, password)
	return &Client{transport: transport, formats: strfmt.Default}
}

// New creates a new timestamp API client with a bearer token for authentication.
//
// It takes the following parameters:
// - host: http host (github.com).
// - basePath: any base path for the API client ("/v1", "/v3").
// - scheme: http scheme ("http", "https").
// - bearerToken: bearer token for Bearer authentication header.
func NewClientWithBearerToken(host, basePath, scheme, bearerToken string) ClientService {
	transport := httptransport.New(host, basePath, []string{scheme})
	transport.DefaultAuthentication = httptransport.BearerToken(bearerToken)
	return &Client{transport: transport, formats: strfmt.Default}
}

// Client for timestamp API.
type Client struct {
	transport runtime.ContextualTransport
	formats   strfmt.Registry
}

// ClientOption may be used to customize the behavior of Client methods.
type ClientOption func(*runtime.ClientOperation)

// This client is generated with a few options you might find useful for your swagger spec.
//
// Feel free to add you own set of options.

// WithContentType allows the client to force the Content-Type header
// to negotiate a specific Consumer from the server.
//
// You may use this option to set arbitrary extensions to your MIME media type.
func WithContentType(mime string) ClientOption {
	return func(r *runtime.ClientOperation) {
		r.ConsumesMediaTypes = []string{mime}
	}
}

// WithContentTypeApplicationJSON sets the Content-Type header to "application/json".
func WithContentTypeApplicationJSON(r *runtime.ClientOperation) {
	r.ConsumesMediaTypes = []string{"application/json"}
}

// WithContentTypeApplicationTimestampQuery sets the Content-Type header to "application/timestamp-query".
func WithContentTypeApplicationTimestampQuery(r *runtime.ClientOperation) {
	r.ConsumesMediaTypes = []string{"application/timestamp-query"}
}

// WithAccept allows the client to force the Accept header
// to negotiate a specific Producer from the server.
//
// You may use this option to set arbitrary extensions to your MIME media type.
func WithAccept(mime string) ClientOption {
	return func(r *runtime.ClientOperation) {
		r.ProducesMediaTypes = []string{mime}
	}
}

// WithAcceptApplicationJSON sets the Accept header to "application/json".
func WithAcceptApplicationJSON(r *runtime.ClientOperation) {
	r.ProducesMediaTypes = []string{"application/json"}
}

// WithAcceptApplicationPemCertificateChain sets the Accept header to "application/pem-certificate-chain".
func WithAcceptApplicationPemCertificateChain(r *runtime.ClientOperation) {
	r.ProducesMediaTypes = []string{"application/pem-certificate-chain"}
}

// WithAcceptApplicationTimestampReply sets the Accept header to "application/timestamp-reply".
func WithAcceptApplicationTimestampReply(r *runtime.ClientOperation) {
	r.ProducesMediaTypes = []string{"application/timestamp-reply"}
}

// ClientService is the interface for Client methods.
type ClientService interface {

	// GetTimestampCertChain retrieve the certificate chain for timestamping that can be used to validate trusted timestamps.
	GetTimestampCertChain(params *GetTimestampCertChainParams, opts ...ClientOption) (*GetTimestampCertChainOK, error)

	// GetTimestampCertChainContext retrieve the certificate chain for timestamping that can be used to validate trusted timestamps.
	GetTimestampCertChainContext(ctx context.Context, params *GetTimestampCertChainParams, opts ...ClientOption) (*GetTimestampCertChainOK, error)

	// GetTimestampResponse generates a new timestamp response and creates a new log entry for the timestamp in the transparency log.
	GetTimestampResponse(params *GetTimestampResponseParams, writer io.Writer, opts ...ClientOption) (*GetTimestampResponseOK, *GetTimestampResponseCreated, error)

	// GetTimestampResponseContext generates a new timestamp response and creates a new log entry for the timestamp in the transparency log.
	GetTimestampResponseContext(ctx context.Context, params *GetTimestampResponseParams, writer io.Writer, opts ...ClientOption) (*GetTimestampResponseOK, *GetTimestampResponseCreated, error)

	SetTransport(transport runtime.ContextualTransport)
}

// GetTimestampCertChain retrieves the certificate chain for timestamping that can be used to validate trusted timestamps.
//
// Returns the certificate chain for timestamping that can be used to validate trusted timestamps.
//
// This method does not support injected context.
// However, timeout and opentracing contexts are honored whenever enabled.
//
// If you need to pass a specific context, use [Client.GetTimestampCertChainContext] instead.
func (a *Client) GetTimestampCertChain(params *GetTimestampCertChainParams, opts ...ClientOption) (*GetTimestampCertChainOK, error) {
	var ctx context.Context
	if params.inner.ctx != nil {
		ctx = params.inner.ctx
	} else {
		ctx = context.Background()
	}

	return a.GetTimestampCertChainContext(ctx, params, opts...)
}

// GetTimestampCertChainContext retrieves the certificate chain for timestamping that can be used to validate trusted timestamps.
//
// Returns the certificate chain for timestamping that can be used to validate trusted timestamps.
//
// Do not use the deprecated [GetTimestampCertChainParams.Context] with this method: it would be ignored.
func (a *Client) GetTimestampCertChainContext(ctx context.Context, params *GetTimestampCertChainParams, opts ...ClientOption) (*GetTimestampCertChainOK, error) {
	// NOTE: parameters are not validated before sending
	if params == nil {
		params = NewGetTimestampCertChainParams()
	}

	op := &runtime.ClientOperation{
		ID:                 "getTimestampCertChain",
		Method:             "GET",
		PathPattern:        "/api/v1/timestamp/certchain",
		ProducesMediaTypes: []string{"application/pem-certificate-chain"},
		ConsumesMediaTypes: []string{"application/json"},
		Schemes:            []string{"http"},
		Params:             params,
		Reader:             &GetTimestampCertChainReader{formats: a.formats},
		Client:             params.HTTPClient,
	}

	for _, opt := range opts {
		opt(op)
	}

	result, err := a.transport.SubmitContext(ctx, op)
	if err != nil {
		return nil, err
	}

	// only one success response has to be checked
	success, ok := result.(*GetTimestampCertChainOK)
	if ok {
		return success, nil
	}

	// unexpected success response.
	//
	// a default response is provided: fill this and return an error
	unexpectedSuccess := result.(*GetTimestampCertChainDefault)

	return nil, runtime.NewAPIError("unexpected success response: content available as default response in error", unexpectedSuccess, unexpectedSuccess.Code())
}

// GetTimestampResponse generates a new timestamp response and creates a new log entry for the timestamp in the transparency log.
//
// This method does not support injected context.
// However, timeout and opentracing contexts are honored whenever enabled.
//
// If you need to pass a specific context, use [Client.GetTimestampResponseContext] instead.
func (a *Client) GetTimestampResponse(params *GetTimestampResponseParams, writer io.Writer, opts ...ClientOption) (*GetTimestampResponseOK, *GetTimestampResponseCreated, error) {
	var ctx context.Context
	if params.inner.ctx != nil {
		ctx = params.inner.ctx
	} else {
		ctx = context.Background()
	}

	return a.GetTimestampResponseContext(ctx, params, writer, opts...)
}

// GetTimestampResponseContext generates a new timestamp response and creates a new log entry for the timestamp in the transparency log.
//
// Do not use the deprecated [GetTimestampResponseParams.Context] with this method: it would be ignored.
func (a *Client) GetTimestampResponseContext(ctx context.Context, params *GetTimestampResponseParams, writer io.Writer, opts ...ClientOption) (*GetTimestampResponseOK, *GetTimestampResponseCreated, error) {
	// NOTE: parameters are not validated before sending
	if params == nil {
		params = NewGetTimestampResponseParams()
	}

	op := &runtime.ClientOperation{
		ID:                 "getTimestampResponse",
		Method:             "POST",
		PathPattern:        "/api/v1/timestamp",
		ProducesMediaTypes: []string{"application/timestamp-reply"},
		ConsumesMediaTypes: []string{"application/timestamp-query", "application/json"},
		Schemes:            []string{"http"},
		Params:             params,
		Reader:             &GetTimestampResponseReader{formats: a.formats, writer: writer},
		Client:             params.HTTPClient,
	}

	for _, opt := range opts {
		opt(op)
	}

	result, err := a.transport.SubmitContext(ctx, op)
	if err != nil {
		return nil, nil, err
	}

	// several success responses have to be checked
	switch value := result.(type) {
	case *GetTimestampResponseOK:
		return value, nil, nil
	case *GetTimestampResponseCreated:
		return nil, value, nil
	}

	// unexpected success response.
	//
	// a default response is provided: fill this and return an error
	unexpectedSuccess := result.(*GetTimestampResponseDefault)
	return nil, nil, runtime.NewAPIError("unexpected success response: content available as default response in error", unexpectedSuccess, unexpectedSuccess.Code())
}

// SetTransport changes the transport on the client
func (a *Client) SetTransport(transport runtime.ContextualTransport) {
	a.transport = transport
}

// innerParams captures internal fields so they don't conflict with user-supplied parameters.
type innerParams struct {
	timeout time.Duration

	// Deprecated: use the operation call with context to pass the context instead of [TimestampParams].
	ctx context.Context
}
