// Copyright 2022 The Sigstore Authors.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package server

import (
	"net/http"
	"time"

	"github.com/prometheus/client_golang/prometheus/promhttp"
)

// NewPrometheusServer creates a server for serving prometheus metrics
func NewPrometheusServer(readTimeout, writeTimeout time.Duration) *http.Server {
	return &http.Server{
		Addr:         ":2112",
		ReadTimeout:  readTimeout,
		WriteTimeout: writeTimeout,
		Handler:      promhttp.Handler(),
	}
}
